/**
 * Example Ratios -- the Ratio class
 *
 * This class encapsulates a numerator and denominator and
 * includes the capability to set the numerator or denominator,
 * retrieve the numerator or denominator, retrieve the decimal
 * equivalent of the ratio, and return a "pretty" String 
 * representation of the ratio
 * 
 * This version includes a non-destructive method to add another
 * ratio to this and return a new ratio that contains the sum
 *
 * Jim Teresco, The College of Saint Rose, CSC 202, Fall 2012
 * CSC 523, Summer 2014, Siena College CSIS 210, Fall 2018
 *
 * @author Jim Teresco
 */

public class Ratio {

    // Instance variables for the two integers, one each for the numerator
    // and denominator.

    private int numerator;
    private int denominator;
    
    // our straightforward constructor
    public Ratio(int num, int den) {
        
        numerator = num;
        denominator = den;
    }
    
    // Mutators for numerator and denominator
    public void setNumerator(int num) {
        
        numerator = num;
    }
    
    public void setDenominator(int den) {
        
        denominator = den;
    }
    
    // And some accessors
    public int getNumerator() {
        
        return numerator;
    }
    
    public int getDenominator() {
        
        return denominator;
    }
    
    public double getDecimalValue() {
        
        return 1.0 * numerator / denominator;
    }
    
    // print our simple string representation
    public String toString() {
        
        return numerator + "/" + denominator;
    }
    
    // a non-destructive add method that adds another ratio to this
    // one and returns a new one (leaving this and other unchanged)
    public Ratio add(Ratio other) {
        
        return new Ratio(numerator*other.denominator + denominator*other.numerator,
                         denominator*other.denominator);
        
    }
    
    // a recursive method to compute greatest common divisor using Euclid's method
    public static int gcd(int a, int b) {
        
        if (a == 0) return b;
        if (a > b) return gcd(b, a);  // so below we can assume b>=a
        return gcd(b%a, a);
    }
    
    // Reduce this ratio to lowest terms.  To do so, we find the greatest common
    // divisor of the numerator and denominator, then divide both
    // numerator and denominator by that value
    public void reduce() {
        
        int divideBy = gcd(numerator, denominator);
        numerator /= divideBy;
        denominator /= divideBy;
    }
}
