/**
 * A recursive data structure (list) of Ratio objects with some
 * useful methods to manage such a list
 *
 * Jim Teresco, The College of Saint Rose, CSC 523, Summer 2014
 * Siena College, CSIS 210, Fall 2018
 *
 * @author Jim Teresco
 */

public class RatioList {

    // our instance variables have room for one Ratio, and
    // then a list of other ratios -- which is exactly
    // the structure we are defining -- so we can define this
    // structure *recursively*
    private Ratio first;
    private RatioList rest;

    /**
       Construct a RatioList from a Ratio and another RatioList,
       which would be null when constructing a brand new RatioList
       with its first element

       @param first the Ratio to become the first element in the list
       @param last the RatioList which will become the rest of the new list
    */
    public RatioList(Ratio first, RatioList rest) {
        
        this.first = first;
        this.rest = rest;
    }

    /**
       compute the sum of all ratios in the list, recursively

       @return a Ratio representing the sum of all Ratios in the list
    */
    public Ratio getSum() {
        
        // if there is no rest, the sum is just first
        if (rest == null) return first;
        
        // otherwise, we have to compute the sum of the rest (using
        // this method) and then add in first, using Ratio's add method
	Ratio restSum = rest.getSum();
	Ratio answer = first.add(restSum);
        return answer;
    }

    /**
       find the minimum ratio, recursively
       
       the minimum is the smaller of first and the minimum of the rest,
       which we compute using this method recursively
       
       @return the Ratio in the list with the smallest value
    */
    public Ratio getMin() {
        
        // the smallest out of a single ratio is that ratio
        if (rest == null) return first;
        
        // otherwise, find the min of the rest and compare to first
        Ratio minOfRest = rest.getMin();
        if (minOfRest.getDecimalValue() < first.getDecimalValue()) {
            return minOfRest;
        }
        else {
            return first;
        }
    }
    
    /**
       find the maximum ratio, recursively
       
       the maximum is the larger of first and the maximum of the rest,
       which we compute using this method recursively
       
       @return the Ratio in the list with the largest value
    */
    public Ratio getMax() {
        
        // the largest out of a single ratio is that ratio
        if (rest == null) return first;
        
        // otherwise, find the max of the rest and compare to first
        Ratio maxOfRest = rest.getMax();
        if (maxOfRest.getDecimalValue() > first.getDecimalValue()) {
            return maxOfRest;
        }
        else {
            return first;
        }
    }
    
    /**
       reduce all Ratios in the list to lowest terms

    */
    public void reduceAll() {
    
        // reduce the one Ratio we have
        first.reduce();
        
        // and if there are more, reduce recursively
        if (rest != null) rest.reduceAll();
    }
    
    /**
       create a String representation of the Ratios in the list

       the toString method is recursive!  To print this whole RatioList,
       we concatenate the result of first's toString with a recursive
       call to this toString method.  But only do the latter when rest
       is not null (we are not at the end of the list)

       @return a String representation of all of the Ratios in the list
    */
    public String toString() {
        
        String answer = first.toString();
        if (rest != null) {
            answer += ", " + rest.toString();
        }
       
        return answer;
    }
}
